"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const MAX_INDEXING_BYTES = 16 * Math.pow(2, 20);
const MAX_INDEXING_COUNT = 1000;
const COMMA = Buffer.from(',');
const OPEN = Buffer.from('{"value":[');
const CLOSE = Buffer.from(']}');
class IndexBuffer {
    constructor(flushHandler) {
        this.flushHandler = flushHandler;
        this.chunks = [];
        this.bytes = 0;
        this.count = 0;
    }
    add(document) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.count) {
                this.chunk(OPEN);
            }
            const next = Buffer.from(JSON.stringify(document));
            if (next.byteLength > MAX_INDEXING_BYTES) {
                throw new Error(`Document #${this.count} contains ${next.byteLength} bytes, which exceeds maximum of ${MAX_INDEXING_BYTES}`);
            }
            else if (next.byteLength + COMMA.byteLength + CLOSE.byteLength + this.bytes > MAX_INDEXING_BYTES) {
                yield this.flush();
            }
            else {
                if (this.count > 0) {
                    this.chunk(COMMA);
                }
                this.chunk(next);
                this.count += 1;
            }
            if (this.count >= MAX_INDEXING_COUNT) {
                yield this.flush();
            }
        });
    }
    flush() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.count) {
                console.log(`Flushing ${this.count} documents`);
                this.chunk(CLOSE);
                yield this.flushHandler(Buffer.concat(this.chunks));
                this.reset();
            }
        });
    }
    chunk(chunk) {
        this.chunks.push(chunk);
        this.bytes += chunk.byteLength;
    }
    reset() {
        this.chunks.length = 0;
        this.bytes = 0;
        this.count = 0;
    }
}
exports.IndexBuffer = IndexBuffer;
//# sourceMappingURL=index-buffer.js.map